@@ -1,4 +1,3 @@
-## Please see file perltidy.ERR
 use v5.10.1;
 
 package NQP::Config::_Scoping;
@@ -37,7 +36,7 @@ $SIG{__DIE__} = sub { confess @_ };
 use base qw<Exporter>;
 our @EXPORT    = qw<rm_l>;
 our @EXPORT_OK = qw<
-  os2platform slash slurp system_or_die run_or_die cmp_rev read_config
+  os2platform slash slurp system_or_die run_or_die cmp_rev read_config read_config_from_command
 >;
 
 # Platform names will be incorporated into a regexp.
@@ -63,6 +62,9 @@ my %platform_vars = (
         windows => ';',
         default => ':',
     },
+    tab => {
+        default => "\t",
+    },
     env_open => {
         windows => '%',
         vms     => '%%',
@@ -127,10 +129,27 @@ sub init {
         jvm  => 'j',
         js   => 'js',
     };
+
+    # Precompiled files extensions
+    $self->{backend_ext} = {
+        moar => 'moarvm',
+        jvm  => 'jar',
+        js   => 'js',
+    };
+
+    # Value of nqp --target
+    $self->{backend_target} = {
+        moar => 'mbc',
+        jvm  => 'jar',
+        js   => 'js',
+    };
     $self->{backends_order} = [qw<moar jvm js>];
-    $self->{options}        = {};
-    $self->{contexts}       = [];
-    $self->{repo_maps}      = {
+    $self->{options}        = {
+        'silent-build' => 1,
+        'clean'        => 1,
+    };
+    $self->{contexts}  = [];
+    $self->{repo_maps} = {
         rakudo => [qw<rakudo rakudo>],
         nqp    => [qw<perl6 nqp>],
         moar   => [qw<MoarVM MoarVM>],
@@ -159,9 +178,11 @@ sub mute {
 sub _gen_msg {
     my $self = shift;
     my $type = shift;
-    my @msg  = ("===$type===\n",
-      join( "\n", map { "  $_" } split /\n/s, join( "", @_ ) ));
-    return wantarray ? @msg : join("", @msg);
+    my @msg  = (
+        "===$type===\n",
+        join( "\n", map { "  $_" } split /\n/s, join( "", @_ ) )
+    );
+    return wantarray ? @msg : join( "", @msg );
 }
 
 sub sorry {
@@ -219,13 +240,38 @@ sub make_cmd {
     if ( $self->is_win ) {
         my $prefix    = $config->{prefix};
         my $has_nmake = 0 == system('nmake /? >NUL 2>&1');
-        my $has_cl    = can_run('cl') && `cl 2>&1` =~ /Microsoft Corporation/;
+        my $cl_report;
+        my $has_cl = can_run('cl')
+          && ( $cl_report = `cl 2>&1` ) =~ /Microsoft Corporation/;
         my $has_gmake = 0 == system('gmake --version >NUL 2>&1');
         my $has_gcc   = 0 == system('gcc --version >NUL 2>&1');
+        if ($has_cl) {
+            if ( $cl_report =~
+                /Microsoft\s.*\sCompiler\s+Version\s+(\d+(?:\.\d+)+)/i )
+            {
+                my $actual_version = $1;
+                my $expect_version = "19.0";
+                if ( version->parse($actual_version) < $expect_version ) {
+                    $self->sorry( "Expected Microsoft Compiler version "
+                          . $expect_version
+                          . "+, but got "
+                          . $actual_version );
+                }
+            }
+        }
         if (
+            -x "$prefix\\bin\\nqp-m.exe"
+            && ( $_ =
+`"$prefix\\bin\\nqp-m.exe" -e "print(nqp::backendconfig()<make>)"`
+            )
+          )
+        {
+            $make = $_;
+        }
+        elsif (
             -x "$prefix\\bin\\nqp-m.bat"
             && ( $_ =
-                `$prefix\\bin\\nqp-m.bat -e "print(nqp::backendconfig()<make>)"`
+`"$prefix\\bin\\nqp-m.bat" -e "print(nqp::backendconfig()<make>)"`
             )
           )
         {
@@ -292,6 +338,16 @@ sub backend_abbr {
     return $self->{backend_prefix}{ $self->validate_backend($backend) };
 }
 
+sub backend_ext {
+    my ( $self, $backend ) = @_;
+    return $self->{backend_ext}{ $self->validate_backend($backend) };
+}
+
+sub backend_target {
+    my ( $self, $backend ) = @_;
+    return $self->{backend_target}{ $self->validate_backend($backend) };
+}
+
 sub backend_config {
     my ( $self, $backend ) = ( shift, shift );
     if (@_) {
@@ -386,47 +442,28 @@ sub configure_paths {
 }
 
 sub configure_jars {
-    my $self    = shift;
+    my $self = shift;
+    my $jars = shift;
+
     my $config  = $self->{config};
     my $options = $self->{options};
-    if ( $options->{'with-asm'} ) {
-        if ( $options->{'with-asm'} ne '-' ) {
-            $config->{'asm'} = $options->{'with-asm'};
-        }
-    }
-    else {
-        $config->{'asm'} = $self->base_path(qw<3rdparty asm asm-4.1.jar>);
-    }
-    if ( $options->{'with-asm-tree'} ) {
-        if ( $options->{'with-asm-tree'} ne '-' ) {
-            $config->{'asmtree'} = $options->{'with-asm-tree'};
-        }
-    }
-    else {
-        $config->{'asmtree'} =
-          $self->base_path(qw<3rdparty asm asm-tree-4.1.jar>);
-    }
-    if ( $options->{'with-jline'} ) {
-        if ( $options->{'with-jline'} ne '-' ) {
-            $config->{'jline'} = $options->{'with-jline'};
-        }
-    }
-    else {
-        $config->{'jline'} = $self->base_path("3rdparty/jline/jline-1.0.jar");
-    }
-    if ( $options->{'with-jna'} ) {
-        if ( $options->{'with-jna'} ne '-' ) {
-            $config->{'jna'} = $options->{'with-jna'};
-        }
-    }
-    else {
-        $config->{'jna'} = $self->base_path("3rdparty/jna/jna-4.0.0.jar");
-    }
 
-    $config->{asmfile} = ( File::Spec->splitpath( $config->{asm} ) )[-1];
-    $config->{jlinefile} =
-      ( File::Spec->splitpath( $config->{jline} ) )[-1];
-    $config->{jnafile} = ( File::Spec->splitpath( $config->{jna} ) )[-1];
+    foreach my $name ( keys %$jars ) {
+        my $path     = $jars->{$name};
+        my $variable = $name;
+        $variable =~ s/-//;
+        if ( $options->{"with-$name"} ) {
+            if ( $options->{"with-$name"} ne '-' ) {
+                $config->{$variable} = $options->{"with-$name"};
+            }
+        }
+        else {
+            $config->{$variable} = $self->base_path( @{$path} );
+        }
+
+        $config->{ $variable . 'file' } =
+          ( File::Spec->splitpath( $config->{$variable} ) )[-1];
+    }
 }
 
 sub configure_relocatability {
@@ -440,6 +477,19 @@ sub configure_relocatability {
         );
     }
 
+    if (
+        $self->{options}->{relocatable}
+        && (   $self->{options}->{'rakudo-home'}
+            || $self->{options}->{'nqp-home'} )
+      )
+    {
+        $self->sorry(
+"It's not possible to build a relocatable rakudo and use hard coded rakudo-home"
+              . "\nor nqp-home directories. So either don't use the `--relocatable` parameter or don't"
+              . "\nuse the `--perl6-home`, `--rakudo-home`, and `--nqp-home` parameters."
+        );
+    }
+
     $self->{config}->{relocatable} =
       $self->{options}->{relocatable} ? 'reloc' : 'nonreloc';
 }
@@ -448,7 +498,7 @@ sub configure_relocatability {
 sub configure_repo_urls {
     my $self = shift;
 
-    # Pre-cach repo urls to make them available for makefiles.
+    # Pre-cache repo urls to make them available for makefiles.
     for my $r ( keys %{ $self->{repo_maps} } ) {
         $self->repo_url( $r, action => 'pull' );
         $self->repo_url( $r, action => 'push' );
@@ -467,23 +517,37 @@ sub configure_commands {
         $ok = run( command => [ $config->{make}, q</?> ], buffer => \$buf );
     }
     if ( $buf =~ /^GNU Make/s ) {
-        $config->{make_family} = 'gnu';
+        $config->{make_family}       = 'gnu';
+        $config->{make_first_prereq} = '$<';
+        $config->{make_all_prereq}   = '$^';
+        $config->{make_pp_pfx} = '';    # make preprocessor directive prefix
     }
-    elsif ( $buf =~ /Microsoft .* NMAKE/s ) {
-        $config->{make_family} = 'nmake';
+    elsif ( $buf =~ /Microsoft \(R\) Program Maintenance Utility/s ) {
+        $config->{make_family}       = 'nmake';
+        $config->{make_first_prereq} = '%s';
+        $config->{make_all_prereq}   = '$**';
+        $config->{make_pp_pfx}       = '!';
     }
     elsif ( $self->is_bsd && $config->{make} =~ /\bmake$/ ) {
-        $config->{make_family} = 'bsd';
+        $config->{make_family}       = 'bsd';
+        $config->{make_first_prereq} = '${>:[1]}';
+        $config->{make_all_prereq}   = '$>';
+        $config->{make_pp_pfx}       = '.';
     }
     unless ( defined $config->{make_family} ) {
         $self->sorry(
-            "Cannot determine the brand of your $config->{make} utility.");
+                "Cannot determine the brand of your $config->{make} utility."
+              . "\nIt is reporting itself as:\n"
+              . $buf );
     }
 
     if ( $self->isa_unix ) {
         $config->{mkpath} = 'mkdir -p --';
         $config->{chmod}  = 'chmod --';
         $config->{cp}     = 'cp --';
+
+        # Symlinking should override destination.
+        $config->{ln_s}   = 'ln -nfs --';
         $config->{rm_f}   = 'rm -f --';
         $config->{rm_rf}  = 'rm -rf --';
         $config->{rm_l}   = 'rm -f --';
@@ -493,6 +557,7 @@ sub configure_commands {
         $config->{mkpath} = '$(PERL5) -MExtUtils::Command -e mkpath';
         $config->{chmod}  = '$(PERL5) -MExtUtils::Command -e chmod';
         $config->{cp}     = '$(PERL5) -MExtUtils::Command -e cp';
+        $config->{ln_s}   = '$(PERL5) -MExtUtils::Command -e cp';
         $config->{rm_f}   = '$(PERL5) -MExtUtils::Command -e rm_f';
         $config->{rm_rf}  = '$(PERL5) -MExtUtils::Command -e rm_rf';
         $config->{rm_l} =
@@ -514,7 +579,25 @@ sub configure_backends {
 }
 
 sub configure_misc {
-    abstract;
+    my $self        = shift;
+    my $config      = $self->{config};
+    my $make_pp_pfx = $self->cfg('make_pp_pfx');
+    if ( $self->cfg('silent_build') eq 'on' ) {
+        $config->{NOECHO_declaration} = <<NOECHO_DECL;
+NOECHO = @
+${make_pp_pfx}ifdef VERBOSE_BUILD 
+NOECHO = 
+${make_pp_pfx}endif
+NOECHO_DECL
+    }
+    else {
+        $config->{NOECHO_declaration} = <<NOECHO_DECL;
+NOECHO = 
+${make_pp_pfx}ifdef SILENT_BUILD 
+NOECHO = @
+${make_pp_pfx}endif
+NOECHO_DECL
+    }
 }
 
 sub configure_refine_vars {
@@ -535,11 +618,6 @@ sub configure_refine_vars {
         $config->{prefix} = $default;
     }
     $config->{prefix} = File::Spec->rel2abs( $config->{prefix} );
-
-    unless ( $config->{libdir} ) {
-        $config->{libdir} =
-          File::Spec->catdir( $config->{prefix}, 'share' );
-    }
 }
 
 sub parse_backends {
@@ -586,18 +664,33 @@ sub configure_from_options {
     my $self   = shift;
     my $config = $self->{config};
     for my $opt (
-        qw<prefix libdir sdkroot sysroot github-user git-protocol
+        qw<prefix rakudo-home nqp-home sdkroot sysroot github-user git-protocol
         rakudo-repo nqp-repo moar-repo roast-repo makefile-timing
-        relocatable reference>
+        relocatable git-cache-dir>
       )
     {
         ( my $ckey = $opt ) =~ s/-/_/g;
         $self->set_key( $ckey, $self->{options}{$opt}, default => '', );
     }
 
+    for my $opt ( keys %{ $self->{options} } ) {
+        my $opt_val = $self->{options}{$opt} // '';
+        next if ref($opt_val);
+        ( my $cf_var = $opt ) =~ s/-/_/g;
+        $config->{"opt_$cf_var"} = $opt_val;
+    }
+
+    if ( $self->{options}{'git-reference'} ) {
+        print "===WARNING!===\n";
+        print "The --git-reference option does not exist anymore.\n";
+        print "Consider using --git-cache-dir instead.\n";
+    }
+
     $config->{stagestats} = '--stagestats'
       if $self->{options}{'makefile-timing'};
 
+    $config->{silent_build} = $self->option('silent-build') ? "on" : "off";
+
     my ( $template, $out );
     if ( $self->option('expand') ) {
         $self->mute;
@@ -683,15 +776,13 @@ sub make_option {
     state $bool_opt = {
         map { $_ => 1 }
           qw<
-          relocatable no-clean ignore-errors
+          force-rebuild relocatable no-clean ignore-errors silent-build
           >
     };
 
     my $opt_str = "";
     if ( $bool_opt->{$opt} ) {
-        if ( $options->{$opt} ) {
-            $opt_str = "--$opt";
-        }
+        $opt_str = "--" . ( $options->{$opt} ? '' : 'no-' ) . "$opt";
     }
     elsif ( defined $options->{$opt} ) {
         my $opt_value =
@@ -712,8 +803,8 @@ sub make_option {
 sub ignorable_opts {
     my $self = shift;
     my $opt  = shift;
-    return qw<gen-moar gen-nqp help make-install expand out
-      prefix backends set-var>;
+    return qw<gen-moar gen-nqp force-rebuild help make-install expand out
+      prefix backends set-var silent-build clean>;
 }
 
 # Generate Configure.pl options from the data we have so far.
@@ -732,7 +823,9 @@ sub opts_for_configure {
         push @subopts, $opt_str if $opt_str;
     }
     push @subopts, "--backends=" . join( ",", $self->active_backends );
-    push @subopts, "--prefix=" . $self->cfg('prefix');
+    push @subopts,
+      "--prefix=" . $self->shell_quote_filename( $self->cfg('prefix') );
+    push @subopts, "--silent-build" if $self->option('silent-build');
     return wantarray ? @subopts : join( " ", @subopts );
 }
 
@@ -772,7 +865,7 @@ sub isa_unix {
 sub is_executable {
     my ( $self, $file ) = @_;
     die "File parameter is missing in call to is_executable" if @_ < 2;
-    return $file if -x $file;
+    return $file                                             if -x $file;
     for my $ext (qw<exe bat>) {
         my $fname = $file . $self->cfg($ext);
         return $fname if -x $fname;
@@ -861,7 +954,7 @@ sub find_filepath {
     my $ctx_subdir = $self->cfg('ctx_subdir');
     push @subdirs, $ctx_subdir if $ctx_subdir;
 
-    my $where = $params{where} || 'templates';
+    my $where     = $params{where} || 'templates';
     my $where_dir = $self->cfg( "${where}_dir", strict => 1 );
     my @suffixes;
     push @suffixes, $params{suffix}        if $params{suffix};
@@ -991,13 +1084,9 @@ sub fill_template_text {
 }
 
 sub reference_dir {
-    my $self      = shift;
-    my $reference = $self->cfg('reference');
-    for my $d (@_) {
-        my $dir = File::Spec->catdir( $reference, $d );
-        return $dir if -d $dir;
-    }
-    return '';
+    my ($self, $name) = @_;
+    my $git_cache_dir = $self->cfg('git_cache_dir');
+    return File::Spec->catdir( $git_cache_dir, $name );
 }
 
 sub git_checkout {
@@ -1009,18 +1098,34 @@ sub git_checkout {
     my $config  = $self->config;
     my $options = $self->{options};
     my $pwd     = cwd();
+    my $pullurl = $self->repo_url( $repo, action => 'pull' );
+    my $pushurl = $self->repo_url( $repo, action => 'push' );
 
+    # Clone / fetch git reference repo
+    if ( $config->{git_cache_dir} ) {
+        my $ref_dir =
+          $self->reference_dir( $self->{repo_maps}{$repo}[1] );
+        if ( !-d $ref_dir ) {
+            my @ref_args = ( 'git', 'clone', '--bare' );
+            push @ref_args, "--depth=$options->{'git-depth'}"
+              if $options->{'git-depth'};
+            push @ref_args, $pullurl, $ref_dir;
+            $self->msg("Cloning reference from $pullurl\n");
+            system_or_die(@ref_args);
+        }
+        else {
+            chdir($ref_dir);
+            system_or_die( 'git', 'fetch' );
+            chdir($pwd);
+        }
+    }
     # get an up-to-date repository
     if ( !-d $dir ) {
-        my ( $pullurl, $pushurl ) = (
-            $self->repo_url( $repo, action => 'pull' ),
-            $self->repo_url( $repo, action => 'push' ),
-        );
         my @args = ( 'git', 'clone' );
-        if ( $config->{reference} ) {
+        if ( $config->{git_cache_dir} ) {
             my $ref_dir =
-              $self->reference_dir( $self->{repo_maps}{$repo}[1], $dir );
-            die "Can't $repo repository directory in $config->{reference}"
+              $self->reference_dir( $self->{repo_maps}{$repo}[1] );
+            die "Can't find $repo reference directory in $config->{git_cache_dir}"
               unless $ref_dir;
             push @args, "--reference=$ref_dir";
         }
@@ -1222,11 +1327,21 @@ sub set {
     my $val    = shift;
     my %params = @_;
 
-    unless ( $params{in_ctx} ) {
-        $self->{config}{$key} = $val;
+    if ( my $prop = $params{in_ctx} ) {
+        my $ctx;
+        if ( $prop eq -1 ) {
+            $ctx = $self->{contexts}[-1];
+        }
+        else {
+            unless ( $ctx = $self->in_ctx($prop) ) {
+                $self->sorry( "No context '$prop' found"
+                      . " while attemtped to set variable $key" );
+            }
+        }
+        $ctx->{configs}[-1]{$key} = $val;
     }
     else {
-        $self->{contexts}[-1]{config}{$key} = $val;
+        $self->{config}{$key} = $val;
     }
 
     return $self;
@@ -1253,10 +1368,11 @@ sub prop {
 sub in_ctx {
     my $self = shift;
     my ( $prop, $val ) = @_;
-    my %params = @_;
 
     for my $ctx ( $self->contexts ) {
-        return $ctx if $ctx->{$prop} eq $val;
+        return $ctx
+          if exists $ctx->{$prop}
+          && ( !defined($val) || ( $ctx->{$prop} eq $val ) );
     }
 
     return 0;
@@ -1421,22 +1537,29 @@ sub cmp_rev {
     $cmp;
 }
 
+sub read_config_from_command {
+    my $command = shift;
+    my %config  = ();
+    local $_;
+    no warnings;
+    if ( open my $CONFIG, '-|', $command ) {
+        while (<$CONFIG>) {
+            if (/^([^\s=]+)=(.*)/) { $config{$1} = $2 }
+        }
+        close($CONFIG);
+    }
+    return %config;
+}
+
 sub read_config {
     my @config_src = @_;
     my %config     = ();
-    local $_;
     for my $file (@config_src) {
-        no warnings;
         if ( !-f $file ) {
             print STDERR "No pre-existing installed file found at $file\n";
             next;
         }
-        if ( open my $CONFIG, '-|', "\"$file\" --show-config" ) {
-            while (<$CONFIG>) {
-                if (/^([^\s=]+)=(.*)/) { $config{$1} = $2 }
-            }
-            close($CONFIG);
-        }
+        %config = read_config_from_command("\"$file\" --show-config");
         last if %config;
     }
     return %config;
