@@ -12,7 +12,7 @@ import org.perl6.nqp.sixmodel.reprs.NativeRefInstance;
 import org.perl6.nqp.sixmodel.reprs.VMArrayInstance;
 
 /**
- * Contains implementation of nqp:: ops specific to Rakudo Perl 6.
+ * Contains implementation of nqp:: ops specific to Rakudo
  */
 @SuppressWarnings("unused")
 public final class RakOps {
@@ -246,7 +246,7 @@ public final class RakOps {
                 return null;
         }
 
-        /* The binder may, for a variety of reasons, wind up calling Perl 6 code and overwriting flatArgs, so it needs to be set at the end to return reliably */
+        /* The binder may, for a variety of reasons, wind up calling Raku code and overwriting flatArgs, so it needs to be set at the end to return reliably */
         tc.flatArgs = args;
         return csd;
     }
@@ -434,7 +434,7 @@ public final class RakOps {
     }
 
     public static SixModelObject getThrower(ThreadContext tc, String type) {
-        SixModelObject exHash = Ops.gethllsym("perl6", "P6EX", tc);
+        SixModelObject exHash = Ops.gethllsym("Raku", "P6EX", tc);
         return exHash == null ? null : Ops.atkey(exHash, type, tc);
     }
 
@@ -623,30 +623,6 @@ public final class RakOps {
         return result;
     }
 
-    public static SixModelObject p6decodelocaltime(long sinceEpoch, ThreadContext tc) {
-        // Get calendar for current local host's timezone.
-        Calendar c = Calendar.getInstance();
-        c.setTimeInMillis(sinceEpoch * 1000);
-
-        // Populate result int array.
-        SixModelObject BOOTIntArray = tc.gc.BOOTIntArray;
-        SixModelObject result = BOOTIntArray.st.REPR.allocate(tc, BOOTIntArray.st);
-        tc.native_i = c.get(Calendar.SECOND);
-        result.bind_pos_native(tc, 0);
-        tc.native_i = c.get(Calendar.MINUTE);
-        result.bind_pos_native(tc, 1);
-        tc.native_i = c.get(Calendar.HOUR_OF_DAY);
-        result.bind_pos_native(tc, 2);
-        tc.native_i = c.get(Calendar.DAY_OF_MONTH);
-        result.bind_pos_native(tc, 3);
-        tc.native_i = c.get(Calendar.MONTH) + 1;
-        result.bind_pos_native(tc, 4);
-        tc.native_i = c.get(Calendar.YEAR);
-        result.bind_pos_native(tc, 5);
-
-        return result;
-    }
-
     public static SixModelObject p6staticouter(SixModelObject code, ThreadContext tc) {
         if (code instanceof CodeRef)
             return ((CodeRef)code).staticInfo.outerStaticInfo.staticCode;
