@@ -154,9 +154,9 @@ sub configure_refine_vars {
 
     $self->SUPER::configure_refine_vars(@_);
 
-    $config->{perl6_home} = $self->nfp(
+    $config->{rakudo_home} = $self->nfp(
         File::Spec->rel2abs(
-            $config->{perl6_home}
+            $config->{rakudo_home}
               || File::Spec->catdir( $config->{'prefix'}, 'share', 'perl6' )
         )
     );
@@ -166,7 +166,7 @@ sub parse_lang_specs {
     my $self = shift;
 
     my $config = $self->{config};
-    my $tmpl   = 'PERL6_SPECS';
+    my $tmpl   = 'RAKU_SPECS';
     open my $sh, "<", $self->template_file_path( $tmpl, required => 1 )
       or self->sorry("Can't open $tmpl: $!");
 
@@ -213,7 +213,7 @@ sub parse_lang_specs {
         }
     }
 
-    $self->{perl6_specs} = \@specs;
+    $self->{raku_specs} = \@specs;
 
     close $sh;
 }
@@ -222,6 +222,8 @@ sub configure_misc {
     my $self   = shift;
     my $config = $self->{config};
 
+    $self->SUPER::configure_misc(@_);
+
     # determine the version of NQP we want
     ( $config->{nqp_want} ) =
       split( ' ',
@@ -237,12 +239,12 @@ sub configure_misc {
     #    return \@elems;
     #};
 
-    #$self->{perl6_specs} = [
+    #$self->{raku_specs} = [
     #    map { $spec_line->($_) }
     #      grep { s/\s*#.*$//; length }
     #      split(
     #        /\n/s,
-    #        slurp( $self->template_file_path( 'PERL6_SPECS', required => 1, ) )
+    #        slurp( $self->template_file_path( 'RAKU_SPECS', required => 1, ) )
     #      )
     #];
 
@@ -317,7 +319,7 @@ sub configure_jvm_backend {
         );
         $config->{'nqp_classpath'} = $nqp_config->{'jvm::runtime.classpath'};
         $config->{'nqp::libdir'}   = $nqp_config->{'nqp::libdir'};
-        $config->{'j_runner'}      = $self->batch_file('perl6-j');
+        $config->{'j_runner'}      = $self->batch_file('rakudo-j');
     }
 }
 
@@ -341,24 +343,24 @@ sub configure_moar_backend {
     }
 
     if ( $config->{relocatable} eq 'reloc' ) {
-        $nqp_config->{static_nqp_home}          = '';
-        $nqp_config->{static_perl6_home}        = '';
-        $nqp_config->{static_nqp_home_define}   = '';
-        $nqp_config->{static_perl6_home_define} = '';
+        $nqp_config->{static_nqp_home}           = '';
+        $nqp_config->{static_rakudo_home}        = '';
+        $nqp_config->{static_nqp_home_define}    = '';
+        $nqp_config->{static_rakudo_home_define} = '';
     }
     else {
         my $qchar = $config->{quote};
-        $nqp_config->{static_nqp_home}   = $nqp_config->{'nqp::nqp_home'};
-        $nqp_config->{static_perl6_home} = $config->{perl6_home};
+        $nqp_config->{static_nqp_home}    = $nqp_config->{'nqp::nqp_home'};
+        $nqp_config->{static_rakudo_home} = $config->{rakudo_home};
         $nqp_config->{static_nqp_home_define} =
             '-DSTATIC_NQP_HOME='
           . $qchar
           . $self->c_escape_string( $nqp_config->{static_nqp_home} )
           . $qchar;
-        $nqp_config->{static_perl6_home_define} =
-            '-DSTATIC_PERL6_HOME='
+        $nqp_config->{static_rakudo_home_define} =
+            '-DSTATIC_RAKUDO_HOME='
           . $qchar
-          . $self->c_escape_string( $nqp_config->{static_perl6_home} )
+          . $self->c_escape_string( $nqp_config->{static_rakudo_home} )
           . $qchar;
     }
 
@@ -434,8 +436,8 @@ sub configure_js_backend {
     $config->{js_nqp} = $ijs->{bin};
     $config->{'perl6_runtime'} =
       File::Spec->rel2abs( $self->nfp('src/vm/js/perl6-runtime') );
-    $config->{'perl6_js_runner'} =
-      $self->batch_file( File::Spec->rel2abs('perl6-js') );
+    $config->{'rakudo_js_runner'} =
+      $self->batch_file( File::Spec->rel2abs('rakudo-js') );
 
     if ( $ijs->{ok} ) {
         $nqp_config = $ijs->{config};
@@ -521,8 +523,8 @@ sub clean_old_p6_libs {
             'NOTICE',
             "Found stale files in $lib_dir.\n",
             "These files were left by a previous install and cause breakage\n",
-            "in this Rakudo version. The installtion will try to remove them\n",
-            "if possible.\n",
+            "in this Rakudo version. The 'install' phase will try to remove\n",
+            "them if possible.\n",
             "\n",
             @notes
         ) if @notes;
@@ -530,9 +532,9 @@ sub clean_old_p6_libs {
 }
 
 # Returns all active language specification entries except for .c
-sub perl6_specs {
+sub raku_specs {
     my $self = shift;
-    return @{ $self->{perl6_specs} };
+    return @{ $self->{raku_specs} };
 }
 
 sub post_active_backends {
@@ -585,18 +587,25 @@ sub gen_nqp {
     my $config  = $self->{config};
 
     #my $nqp_bin      = $options->{'with-nqp'};
-    my $nqp_git_spec = $options->{'gen-nqp'};
-    my $gen_nqp      = $options->{'gen-nqp'};
-    my $gen_moar     = $options->{'gen-moar'};
-    my $prefix       = $config->{prefix};
-    my $sdkroot      = $config->{'sdkroot'};
-    my $startdir     = $config->{'base_dir'};
-    my $exe          = $config->{exe};
-    my $nqp_want     = $config->{nqp_want};
-    my $git_protocol = $options->{'git-protocol'} // 'https';
-    my @moar_options = @{ $options->{'moar-option'} // [] };
-    my $impls        = $self->{impls};
-    my $pwd          = cwd;
+    my $nqp_git_spec  = $options->{'gen-nqp'};
+    my $gen_nqp       = $options->{'gen-nqp'};
+    my $gen_moar      = $options->{'gen-moar'};
+    my $force_rebuild = $options->{'force-rebuild'};
+    my $prefix        = $config->{prefix};
+    my $sdkroot       = $config->{'sdkroot'};
+    my $startdir      = $config->{'base_dir'};
+    my $exe           = $config->{exe};
+    my $nqp_want      = $config->{nqp_want};
+    my $git_protocol  = $options->{'git-protocol'} // 'https';
+    my @moar_options  = @{ $options->{'moar-option'} // [] };
+    my $impls         = $self->{impls};
+    my $pwd           = cwd;
+
+    if ( $force_rebuild && !defined $gen_nqp ) {
+        $self->note( "WARNING",
+                "Command line option --force-rebuild"
+              . " is ineffective without --gen-nqp" );
+    }
 
     my %need;
 
@@ -619,12 +628,16 @@ sub gen_nqp {
           $nqp_have ? ( 0 <= cmp_rev( $nqp_have, $nqp_want ) ) : 0;
         my $nqp_ok = $nqp_have && $nqp_ver_ok;
 
-        unless ( !$nqp_have || $nqp_ver_ok || $options->{'ignore-errors'} ) {
+        unless ( $force_rebuild
+            || !$nqp_have
+            || $nqp_ver_ok
+            || $options->{'ignore-errors'} )
+        {
             $self->note( "WARNING",
                 "$bin version $nqp_have is outdated, $nqp_want expected.\n" );
         }
 
-        if ( $nqp_ok or $options->{'ignore-errors'} ) {
+        if ( !$force_rebuild and ( $nqp_ok or $options->{'ignore-errors'} ) ) {
             $impls->{$b}{ok} = 1;
         }
         elsif ( $self->opt('with-nqp') ) {
@@ -641,7 +654,16 @@ sub gen_nqp {
 
     if ( defined $gen_nqp || defined $gen_moar ) {
         my $user = $options->{'github-user'} // 'perl6';
-        $self->git_checkout( 'nqp', 'nqp', $nqp_git_spec || $nqp_want );
+
+        # Don't expect any specific default commit in nqp/ if the repo is
+        # already checked out and we're force-rebuilding.
+        my $expected_spec =
+          ( $force_rebuild
+              && -d File::Spec->catdir( $self->cfg('base_dir'), 'nqp', '.git' )
+          )
+          ? undef
+          : $nqp_want;
+        $self->git_checkout( 'nqp', 'nqp', $nqp_git_spec || $expected_spec );
     }
 
     my @cmd = (
@@ -651,14 +673,19 @@ sub gen_nqp {
 
     # Append only the options we'd like to pass down to NQP's Configure.pl
     for my $opt (
-        qw<git-depth git-reference github-user nqp-repo moar-repo
-        nqp-home relocatable ignore-errors with-moar silent-build>
+        qw<git-depth github-user nqp-repo moar-repo
+        nqp-home relocatable ignore-errors with-moar silent-build
+        force-rebuild>
       )
     {
         my $opt_str = $self->make_option( $opt, no_quote => 1 );
         push @cmd, $opt_str if $opt_str;
     }
 
+    push @cmd, '--git-cache-dir='
+        . File::Spec->rel2abs($options->{'git-cache-dir'})
+        if $options->{'git-cache-dir'};
+
     push @cmd, "--backends=" . join( ",", $self->active_backends );
 
     if ( defined $gen_moar ) {
@@ -693,7 +720,7 @@ sub _specs_iterate {
 
     $self->not_in_context( specs => 'spec' );
 
-    for my $spec ( $cfg->perl6_specs ) {
+    for my $spec ( $cfg->raku_specs ) {
         my $spec_char   = $spec->[0];
         my $spec_subdir = "6.$spec_char";
         my %config      = (
